//-----------------------------------------------------
// FastSleep.cpp
// Hector Santos, Santronics Software, Inc.
// http://www.santronics.com
// email: sant9442@gmail.com
// email: hsantos@santronics.com
//
// Utility to toggle the PC with Windows OS multi-media
// resolution that speeds up the Sleep() quantum.
//
// All PC has a quantum of 10-15ms regardless of the
// sleep value. Assuming the PC is 15ms, if you use
// Sleep(1), the time slice is 15ms. Sleep(18) is 30ms,
// and so on.
//
// This utility allows you to toggle the resolution
// to 1 ms.  The actual delay is shown by this utility.
//
// Why?
//
// This can change the behavior of your applications
// which depends on timers, like many of the Web 2.0
// Javascript applications.  In Windows, the multi media
// function timeBeginPeriod() changes the resolution
// for the entire system.  So if some other application
// is running with timeBeginPeriod(1) enabled, this
// will have a drastic effect on your Web 2.0 application.
// It may behave better, it may behave unexpected worst.
// It might explain someone reported to you, "hey this
// is not running right."  But others don't see this,
// and you can repeat it, so you just blew the report
// away.    This utility can be use to design your
// application properly under all conditions.
//
// Command line compile with MSVC 6.0:
//
// cl fastsleep.cpp /W3 /GX /MD /D "_AFXDLL"
//
//-----------------------------------------------------

#include <stdio.h>
#include <afx.h>
#include <conio.h>
#include <Mmsystem.h>
#pragma comment(lib,"Winmm.lib")

#define  ThreadSleepTime           1
#define  ThreadResolution          1
#define  Iterations                100

/*
  METHOD0 - ave of start/finish sleep loop tics
  METHOD1 - ave of accumulated loop sleep tics
  METHOD2 - MM, ave of accumulated loop sleep timer
  METHOD3 - MM, ave of start/finish sleep loop timer
*/

#define METHOD2

#ifdef METHOD0
inline double method()
{
   DWORD start=GetTickCount();
   register i;
   for(i=0;i<Iterations;i++) Sleep(ThreadSleepTime);
   DWORD end=GetTickCount();
   double fTot  = end-start;
   double fDiff = (fTot /Iterations*1.0);
   return fDiff;
}
#endif

#ifdef METHOD1
inline double method()
{
   double fTot = 0.0;
   for (int n=0;n<Iterations;n++) {
      DWORD t1 = GetTickCount();
      Sleep(ThreadSleepTime);
      fTot += (GetTickCount()-t1);
   }
   double fDiff = (fTot /Iterations*1.0);
   return fDiff;
}
#endif

#ifdef METHOD2
inline double method()
{
   LARGE_INTEGER start_ticks;
   LARGE_INTEGER ends_ticks;
   LARGE_INTEGER frequency;
   double fTot = 0.0;
   QueryPerformanceFrequency(&frequency);
   for (int n=0;n<Iterations;n++) {
      QueryPerformanceCounter(&start_ticks);
      Sleep(ThreadSleepTime);
      QueryPerformanceCounter(&ends_ticks);
      ends_ticks.QuadPart -= start_ticks.QuadPart;
      fTot += ends_ticks.QuadPart;
   }
   double fDiff = (fTot * 1000.0/ frequency.QuadPart) / (Iterations);
   return fDiff;
}
#endif

#ifdef METHOD3
inline double method()
{
   LARGE_INTEGER start_ticks;
   LARGE_INTEGER ends_ticks;
   LARGE_INTEGER frequency;
   double fTot = 0.0;
   QueryPerformanceFrequency(&frequency);
   QueryPerformanceCounter(&start_ticks);
   for (int n=0;n<Iterations;n++) Sleep(ThreadSleepTime);
   QueryPerformanceCounter(&ends_ticks);
   ends_ticks.QuadPart -= start_ticks.QuadPart;
   fTot = ends_ticks.QuadPart * 1000.0 / frequency.QuadPart;
   double fDiff = fTot / (Iterations*1.0);
   return fDiff;
}
#endif


int main(int argc, char* argv[])
{

   cprintf("- Sleep Interval: %d\n",ThreadSleepTime);
   cprintf("- Interations   : %d\n",Iterations);
   cprintf("- Press ESC to exit, M to toggle 1ms resolution\n");
   int fast = 0;
   while(1)
   {
      if (kbhit()) {
         int ch = getch();
         if (ch == 27) break;
         if (ch == 'M' || ch == 'm') {
            if (fast) {
               timeEndPeriod(ThreadResolution);
               fast = 0;
            } else {
               timeBeginPeriod(ThreadResolution);
               fast = 1;
            }
         }
      }
      double fDiff = method();
      cprintf("%s: %6.2f ms passed!\r",fast?"fast":"----",fDiff);
   }
   cprintf("\n");
   return 0;
}
